<?php
/*
Plugin Name: Urly.one
Description: Generate and manage short URLs for pages and posts using a custom API
Version: 1.1
Author: Ihan Sadiq
Author URI: https://ihansadiq.nl
License: GPL-2.0+
License URI: http://www.gnu.org/licenses/gpl-2.0.txt
*/

if (!defined('ABSPATH')) {
    exit;
}

class urly_one_wp {
    private $api_url = 'https://urly.one/api.php';

    public function __construct() {
        add_action('admin_init', array($this, 'add_custom_columns'));
        add_action('manage_posts_custom_column', array($this, 'custom_column_content'), 10, 2);
        add_action('manage_pages_custom_column', array($this, 'custom_column_content'), 10, 2);
        add_action('admin_footer', array($this, 'add_custom_bulk_actions'));
        add_action('admin_footer', array($this, 'add_inline_js'));
        add_action('wp_ajax_generate_short_url', array($this, 'generate_short_url_ajax'));
        add_action('wp_ajax_custom_short_url', array($this, 'custom_short_url_ajax'));
        add_action('load-edit.php', array($this, 'handle_bulk_action'));
        add_action('admin_notices', array($this, 'bulk_action_admin_notice'));
    }

    public function add_custom_columns() {
        add_filter('manage_posts_columns', array($this, 'add_short_url_column'));
        add_filter('manage_pages_columns', array($this, 'add_short_url_column'));
    }

    public function add_short_url_column($columns) {
        $columns['short_url'] = 'Short URL';
        return $columns;
    }

    public function custom_column_content($column_name, $post_id) {
        if ($column_name == 'short_url') {
            $short_url = get_post_meta($post_id, '_short_url', true);
            if ($short_url) {
                echo '<a href="' . esc_url($short_url) . '" target="_blank">' . esc_html($short_url) . '</a>';
                echo ' <a href="#" class="custom-short-url" data-post-id="' . esc_attr($post_id) . '">Custom</a>';
            } else {
                echo '<a href="#" class="generate-short-url" data-post-id="' . esc_attr($post_id) . '">Generate</a>';
            }
        }
    }

    public function add_custom_bulk_actions() {
        global $post_type;

        if ($post_type == 'post' || $post_type == 'page') {
            ?>
            <script type="text/javascript">
                jQuery(document).ready(function($) {
                    $('<option>').val('generate_short_urls').text(<?php echo wp_json_encode(__('Generate Short URLs', 'urly-one')); ?>).appendTo("select[name='action'], select[name='action2']");
                });
            </script>
            <?php
        }
    }

    public function add_inline_js() {
        ?>
        <script type="text/javascript">
            jQuery(document).ready(function($) {
                $('.generate-short-url, .custom-short-url').on('click', function(e) {
                    e.preventDefault();
                    var postId = $(this).data('post-id');
                    var isCustom = $(this).hasClass('custom-short-url');
                    var customSlug = '';
                    
                    if (isCustom) {
                        customSlug = prompt("Enter custom slug (leave empty for random):");
                        if (customSlug === null) return; // User cancelled
                    }

                    var data = {
                        action: isCustom ? 'custom_short_url' : 'generate_short_url',
                        post_id: postId,
                        custom_slug: customSlug,
                        nonce: '<?php echo esc_js(wp_create_nonce('short_url_nonce')); ?>'
                    };

                    $.post(ajaxurl, data, function(response) {
                        if (response.success) {
                            var cell = $('tr#post-' + postId + ' td.short_url');
                            cell.html('<a href="' + response.data.short_url + '" target="_blank">' + response.data.short_url + '</a> <a href="#" class="custom-short-url" data-post-id="' + postId + '">Custom</a>');
                        } else {
                            alert('Error: ' + response.data.message);
                        }
                    });
                });
            });
        </script>
        <?php
    }

    public function generate_short_url($url, $custom_slug = '') {
        $body = array('url' => $url);
        if (!empty($custom_slug)) {
            $body['custom_url'] = $custom_slug;
        }

        $response = wp_remote_post($this->api_url, array(
            'body' => wp_json_encode($body),
            'headers' => array('Content-Type' => 'application/json'),
        ));

        if (is_wp_error($response)) {
            return false;
        }

        $body = json_decode(wp_remote_retrieve_body($response), true);
        return isset($body['short_url']) ? $body['short_url'] : false;
    }

    public function generate_short_url_ajax() {
        check_ajax_referer('short_url_nonce', 'nonce');
        
        if (!current_user_can('edit_posts')) {
            wp_send_json_error(array('message' => 'Permission denied'));
        }

        $post_id = isset($_POST['post_id']) ? intval($_POST['post_id']) : 0;
        if (!$post_id) {
            wp_send_json_error(array('message' => 'Invalid post ID'));
        }

        $short_url = $this->generate_short_url(get_permalink($post_id));

        if ($short_url) {
            update_post_meta($post_id, '_short_url', $short_url);
            wp_send_json_success(array('short_url' => $short_url));
        } else {
            wp_send_json_error(array('message' => 'Failed to generate short URL'));
        }
    }

    public function custom_short_url_ajax() {
        check_ajax_referer('short_url_nonce', 'nonce');
        
        if (!current_user_can('edit_posts')) {
            wp_send_json_error(array('message' => 'Permission denied'));
        }

        $post_id = isset($_POST['post_id']) ? intval($_POST['post_id']) : 0;
        if (!$post_id) {
            wp_send_json_error(array('message' => 'Invalid post ID'));
        }

        $custom_slug = isset($_POST['custom_slug']) ? sanitize_text_field(wp_unslash($_POST['custom_slug'])) : '';
        $short_url = $this->generate_short_url(get_permalink($post_id), $custom_slug);

        if ($short_url) {
            update_post_meta($post_id, '_short_url', $short_url);
            wp_send_json_success(array('short_url' => $short_url));
        } else {
            wp_send_json_error(array('message' => 'Failed to generate custom short URL'));
        }
    }

    public function handle_bulk_action() {
        $wp_list_table = _get_list_table('WP_Posts_List_Table');
        $action = $wp_list_table->current_action();
    
        if ($action === 'generate_short_urls') {
            check_admin_referer('bulk-posts');
    
            $post_ids = isset($_REQUEST['post']) ? array_map('intval', $_REQUEST['post']) : array();
    
            foreach ($post_ids as $post_id) {
                $short_url = $this->generate_short_url(get_permalink($post_id));
                if ($short_url) {
                    update_post_meta($post_id, '_short_url', $short_url);
                }
            }
    
            $sendback = add_query_arg(
                array(
                    'bulk_short_urls_generated' => count($post_ids),
                    '_wpnonce' => wp_create_nonce('bulk-posts')
                ), 
                wp_get_referer()
            );
            wp_safe_redirect($sendback);
            exit;
        }
    }

    public function bulk_action_admin_notice() {
        $nonce = isset($_REQUEST['_wpnonce']) ? sanitize_key(wp_unslash($_REQUEST['_wpnonce'])) : '';
        if (!wp_verify_nonce($nonce, 'bulk-posts')) {
            return;
        }
    
        if (!empty($_REQUEST['bulk_short_urls_generated'])) {
            $count = isset($_REQUEST['bulk_short_urls_generated']) ? intval($_REQUEST['bulk_short_urls_generated']) : 0;
            $message = sprintf(
                /* translators: %s: number of items */
                _n(
                    'Generated short URL for %s item.',
                    'Generated short URLs for %s items.',
                    $count,
                    'urly-one'
                ),
                number_format_i18n($count)
            );
            echo '<div class="updated"><p>' . esc_html($message) . '</p></div>';
        }
    }
}

$urly_one_wp = new urly_one_wp();